@@ -50,7 +50,7 @@ def manifest(ctx, config): # use as `schematic manifest ...`
 @click.option('-s', '--sheet_url', is_flag=True, help=query_dict(manifest_commands, ("manifest", "get", "sheet_url")))
 @click.option('-o', '--output_csv', help=query_dict(manifest_commands, ("manifest", "get", "output_csv")))
 @click.option('-a', '--use_annotations', is_flag=True, help=query_dict(manifest_commands, ("manifest", "get", "use_annotations")))
-@click.option('-a', '--oauth', is_flag=True, help=query_dict(manifest_commands, ("manifest", "get", "oauth")))
+@click.option('-oa', '--oauth', is_flag=True, help=query_dict(manifest_commands, ("manifest", "get", "oauth")))
 @click.option('-j', '--json_schema', help=query_dict(manifest_commands, ("manifest", "get", "json_schema")))
 @click.pass_obj
 def get_manifest(ctx, title, data_type, jsonld, dataset_id, sheet_url,
